@@ -35,22 +35,10 @@ spl3D <- function(x1,
          "are not in the data:\n", paste0(missVars, collapse = ", "), "\n",
          call. = FALSE)
   }
-  if (!is.numeric(x1lim) || length(x1lim) != 2 ||
-      x1lim[1] > range(x1)[1] || x1lim[2] < range(x1)[2]) {
-    stop("x1lim should be a vector of length two with all values of ", x1Name,
-         " between its lower and upper value.\n")
-  }
-  if (!is.numeric(x2lim) || length(x2lim) != 2 ||
-      x2lim[1] > range(x2)[1] || x2lim[2] < range(x2)[2]) {
-    stop("x2lim should be a vector of length two with all values of ", x2Name,
-         " between its lower and upper value.\n")
-  }
-  if (!is.numeric(x3lim) || length(x3lim) != 2 ||
-      x3lim[1] > range(x3)[1] || x3lim[2] < range(x3)[2]) {
-    stop("x3lim should be a vector of length two with all values of ", x3Name,
-         " between its lower and upper value.\n")
-  }
-  knots <- list()
+  checkLim(lim = x1lim, limName = "x1lim", x = x1, xName = x1Name)
+  checkLim(lim = x2lim, limName = "x2lim", x = x2, xName = x2Name)
+  checkLim(lim = x3lim, limName = "x3lim", x = x3, xName = x3Name)
+  knots <- vector(mode = "list", length = 3)
   knots[[1]] <- PsplinesKnots(x1lim[1], x1lim[2], degree = degree, nseg = nseg[1])
   knots[[2]] <- PsplinesKnots(x2lim[1], x2lim[2], degree = degree, nseg = nseg[2])
   knots[[3]] <- PsplinesKnots(x3lim[1], x3lim[2], degree = degree, nseg = nseg[3])
