@@ -11,12 +11,7 @@
 RowKronecker <- function(X1,
                          X2) {
   if (inherits(X1, "spam") && inherits(X2, "spam")) {
-    L <- RowKron(X1, X2)
-    rowKron <- spam::spam(x = 0, nrow = 0, ncol = 0)
-    rowKron@entries <- L$entries
-    rowKron@colindices <- L$colindices
-    rowKron@rowpointers <- L$rowpointers
-    rowKron@dimension <- L$dimension
+    rowKron <- RowKron(X1, X2)
   } else {
     one.1 <- matrix(1, 1, ncol(X1))
     one.2 <- matrix(1, 1, ncol(X2))
